from typing import List, Optional

from pydantic import BaseModel, EmailStr


class User(BaseModel):
    externalId: str
    email: Optional[EmailStr] = None
    name: Optional[str] = None
    username: Optional[str] = None
    segments: Optional[dict[str, List[str]]] = {}

class UserSegmentTypeResponse(BaseModel):
    id: int
    name: str

class UserSegmentRespone(BaseModel):
    id: int
    type: UserSegmentTypeResponse
    name: str

class UserResponse(BaseModel):
    id: int
    externalId: str
    email: Optional[str] = None
    name: Optional[str] = None
    username: Optional[str] = None
    segments: List[UserSegmentRespone]


from datetime import datetime
from typing import List, Optional, Union

from pydantic import BaseModel, Extra

from melodi.messages.data_models import Message, MessageWithFeedback
from melodi.users.data_models import User, UserResponse


class Thread(BaseModel):
    id: Optional[int] = None
    externalId: Optional[str] = None
    projectId: Optional[int] = None
    projectName: Optional[str] = None
    messages: List[Message]
    metadata: dict[str, Union[str, int]] = {}
    externalUser: Optional[User] = None

class ThreadsQueryParams(BaseModel):
  class Config:
    extra = Extra.forbid

  pageSize: int = 50
  pageIndex: int = 0
  projectId: Optional[int] = None
  ids: Optional[List[int]] = None
  externalIds: Optional[List[str]] = None
  userSegmentIds: Optional[List[int]] = None
  search: Optional[str] = None
  hasFeedback: Optional[bool] = None
  includeFeedback: Optional[bool] = None
  before: Optional[datetime] = None
  after: Optional[datetime] = None

class SimpleProject(BaseModel):
    id: int
    name: str

class ThreadResponse(BaseModel):
    id: int
    organizationId: int
    externalId: Optional[str] = None

    project: SimpleProject

    externalUser: Optional[UserResponse] = None

    metadata: dict[str, Union[str, int]] = {}

    createdAt: datetime
    updatedAt: datetime

class ThreadsPagedResponse(BaseModel):
    count: int
    rows: List[ThreadResponse]

class ThreadResponseWithFeedback(ThreadResponse):
    messages: List[MessageWithFeedback]

class ThreadsWithFeedbackPagedResponse(BaseModel):
    count: int
    rows: List[ThreadResponseWithFeedback]



from melodi.messages.data_models import Message
from melodi.threads.data_models import Thread
from melodi.users.data_models import User


def thread_object_from_open_ai_messages_response(messages_response, project_id: int, user: User = None, metadata: dict = {}) -> Thread:
  messages = messages_response.data

  melodi_messages = []
  for index, message in enumerate(messages):
    if (index == 0):
      thread_id = message.thread_id
    melodi_messages.append(
        Message(
            externalId=message.id,
            role=message.role,
            content=message.content[0].text.value,
        )
    )

  melodi_messages.reverse()

  thread = Thread(
      projectId=project_id,
      externalId=thread_id,
      messages=melodi_messages,
      metadata=metadata,
      externalUser=user,
  )

  return thread


from logging import Logger

from requests import Response


def _log_melodi_http_errors(logger: Logger, response: Response):
    if (response.status_code == 400):
        try:
            responseJson = response.json()
            if ("errors" in responseJson):
                for error in responseJson["errors"]:
                    logger.error(f"Bad Request response from Melodi API: {error}")
            else:
                logger.error(f"Bad Request response from Melodi API: {responseJson}")
        except:
            pass


from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class ProjectResponse(BaseModel):
    id: int
    name: str
    organizationId: int
    userId: Optional[int] = None
    isDefault: bool
    isDeleted: bool
    chainId: Optional[int] = None
    chainDisplayOrder: Optional[int] = None
    useCase: Optional[str] = None
    notes: Optional[str] = None
    createdAt: datetime
    updatedAt: datetime


from typing import Any, List, Literal, Optional, Union

from pydantic import BaseModel

from melodi.feedback.data_models import FeedbackResponse


class IssueMessageAssociation(BaseModel):
    id: int
    issueId: int
    messageId: int
    userId: int

class IntentMessageAssociation(BaseModel):
    id: int
    intentId: int
    messageId: int
    userId: int

class FeedbackMessageAssociation(BaseModel):
    externalFeedback: FeedbackResponse

class Message(BaseModel):
    externalId: Optional[str] = None
    type: Literal['markdown', 'json'] = 'markdown'
    role: str
    content: Optional[str] = None
    jsonContent: Optional[Any] = None
    metadata: dict[str, Union[str, int]] = {}

class MessageWithFeedback(Message):
    externalFeedbackAssociations: List[FeedbackMessageAssociation]

class MessageResponse(Message):
    id: int
    issueAssociations: List[IssueMessageAssociation]
    intentAssociations: List[IntentMessageAssociation]


import logging

import requests
from pydantic import parse_obj_as

from melodi.base_client import BaseClient
from melodi.exceptions import MelodiAPIError
from melodi.logging import _log_melodi_http_errors
from melodi.messages.data_models import (IntentMessageAssociation,
                                         IssueMessageAssociation,
                                         MessageResponse)


class MessagesClient(BaseClient):
    def __init__(self, base_url: str, api_key: str):
        self.api_key = api_key
        self.base_url = base_url

        self.base_endpoint = (
            self.base_url + "/api/external/messages"
        )
        self.endpoint = (
            self.base_endpoint + f"?apiKey={self.api_key}"
        )

        self.issue_message_associations_base_endpoint = self.base_url + "/api/external/issue-message-associations"
        self.issue_message_associations_endpoint = self.issue_message_associations_base_endpoint + f"?apiKey={self.api_key}"

        self.intent_message_associations_base_endpoint = self.base_url + "/api/external/intent-message-associations"
        self.intent_message_associations_endpoint = self.intent_message_associations_base_endpoint + f"?apiKey={self.api_key}"


        self.logger = logging.getLogger(__name__)

    def get(self, message_id: int) -> MessageResponse:
        url = f"{self.base_endpoint}/{message_id}?apiKey={self.api_key}"

        try:
            response = requests.request("GET", url)

            _log_melodi_http_errors(self.logger, response)
            response.raise_for_status()
            return parse_obj_as(MessageResponse, response.json())
        except MelodiAPIError as e:
            raise MelodiAPIError(e)

    def add_issue_to_message(self, issue_id: int, message_id: int) -> IssueMessageAssociation:
        url = self.issue_message_associations_endpoint

        try:
            response = requests.post(
                url, headers=self._get_headers(), json={"issueId": issue_id, "messageId": message_id}
            )

            _log_melodi_http_errors(self.logger, response)
            response.raise_for_status()
            return response.json()
        except MelodiAPIError as e:
            raise MelodiAPIError(e)

    def remove_issue_from_message(self, issue_id: int, message_id: int) -> None:
        message = self.get(message_id)

        for issue_association in message.issueAssociations:
            if (issue_association.issueId == issue_id):
                issue_message_association_id = issue_association.id
                break

        if not issue_message_association_id:
            raise MelodiAPIError(f"Issue {issue_id} is not associated to message {message_id}")

        url = f"{self.issue_message_associations_base_endpoint}/{issue_message_association_id}?apiKey={self.api_key}"

        try:
            response = requests.delete(
                url, headers=self._get_headers()
            )

            _log_melodi_http_errors(self.logger, response)
            response.raise_for_status()
        except MelodiAPIError as e:
            raise MelodiAPIError(e)

    def add_intent_to_message(self, intent_id: int, message_id: int) -> IntentMessageAssociation:
        url = self.intent_message_associations_endpoint

        try:
            response = requests.post(
                url, headers=self._get_headers(), json={"intentId": intent_id, "messageId": message_id}
            )

            _log_melodi_http_errors(self.logger, response)
            response.raise_for_status()
            return response.json()
        except MelodiAPIError as e:
            raise MelodiAPIError(e)

    def remove_intent_from_message(self, intent_id: int, message_id: int) -> None:
        message = self.get(message_id)

        for intent_association in message.intentAssociations:
            if (intent_association.intentId == intent_id):
                intent_message_association_id = intent_association.id
                break

        if not intent_message_association_id:
            raise MelodiAPIError(f"Intent {intent_id} is not associated to log {message_id}")

        url = f"{self.intent_message_associations_base_endpoint}/{intent_message_association_id}?apiKey={self.api_key}"

        try:
            response = requests.delete(
                url, headers=self._get_headers()
            )

            _log_melodi_http_errors(self.logger, response)
            response.raise_for_status()
        except MelodiAPIError as e:
            raise MelodiAPIError(e)




from typing import Dict, List, Optional

from pydantic import BaseModel


class Sample(BaseModel):
    title: Optional[str] = None
    message: Optional[str] = None
    response: str

class Samples(BaseModel):
    samples: List[Sample]

class ComparisonSample(BaseModel):
    title: Optional[str] = None
    message: Optional[str] = None
    response: str
    version: str

class Comparisons(BaseModel):
    samples: List[ComparisonSample]

class BinarySample(BaseModel):
    response: str
    title: Optional[str] = None

class BakeoffSample(BaseModel):
    version: str
    response: str
    message: Optional[str] = None
    title: Optional[str] = None

class Item(BaseModel):
    projectName: str
    versionName: str
    data: Dict



import json
import logging
import re
from typing import Optional

import requests

from melodi.base_client import BaseClient
from melodi.exceptions import MelodiAPIError
from melodi.experiments.data_models import (BakeoffSample, BinarySample,
                                            Comparisons, Samples)


class ExperimentsClient(BaseClient):
    def __init__(self, base_url: str, api_key: str):
        self.api_key = api_key
        self.base_url = base_url

        self.base_endpoint = (
            self.base_url + "/api/external/experiments"
        )
        self.endpoint = (
            self.base_endpoint + f"?apiKey={self.api_key}"
        )

        self.logger = logging.getLogger(__name__)

    def _send_create_experiment_request(self, request_data):
        response = None

        try:
            response = requests.post(
                self.endpoint,
                headers=self._get_headers(),
                json=request_data,
            )
            response.raise_for_status()
            self.logger.info("Successfully created Melodi experiment.")

        except requests.exceptions.RequestException as e:
            self.logger.error(f"Failed to create Melodi experiment: {e}")
            self.logger.error(f"Response: {response.json()}")

        if response and response.status_code == 200:
            try:
                feedback_url = response.json().get("feedbackUrl")
                match = re.search(r"(\d+)$", feedback_url)
                exp_id = int(match.group(1))
                self.logger.info(
                    f"Experiment ID: {exp_id}",
                )
                return {"feedbackUrl": feedback_url, "experimentId": exp_id}
            except MelodiAPIError as e:
                raise MelodiAPIError(f"{e}")
        else:
            self.logger.error("Failed to extract experiment ID")



    def load_samples(self, file_path: str, experiment_type: str) -> list:
        res = []
        self.logger.info(msg=f"Loading samples from: {file_path}")

        if experiment_type == "binary":
            self.logger.info(f"Experiment type = binary")
            with open(file_path, "r") as file:
                for line in file:
                    json_object = json.loads(line.strip())

                    if "response" not in json_object:
                        raise Exception(
                            f'Sample {json_object} is missing "response" ' f"attribute."
                        )

                    res.append(json_object)

        elif experiment_type == "bake_off":
            self.logger.info(f"Experiment type = bake-off")
            with open(file_path, "r") as file:
                for line in file:
                    json_object = json.loads(line.strip())
                    samples = json_object["samples"]

                    for sample in samples:
                        if "response" not in sample:
                            raise Exception(
                                f'Sample {sample} is missing "response" ' f"attribute."
                            )

                        elif "version" not in sample:
                            raise Exception(
                                f'Sample {sample} is missing "version" ' f"attribute."
                            )

                    res.append(json_object)

        self.logger.info(msg=f"Loaded {len(res)} samples")

        return res

    def get_experiments(self):
        return requests.request("GET", self.endpoint)

    def create_experiment(self, name: str, instructions: str, project: str):
        request_data = {
            "experiment": {
                "name": name,
                "instructions": instructions,
                "project": project,
            }
        }

        return self._send_create_experiment_request(request_data=request_data)

    def create_experiment_with_samples(
        self,
        name: str,
        samples: Samples,
        instructions: str = None,
        project: str = None,
        version: str = None,
    ):
        request_data = {
            "experiment": {
                "name": name,
                "instructions": instructions,
                "project": project,
                "version": version
            },
            "samples": samples
        }

        return self._send_create_experiment_request(request_data=request_data)

    def create_experiment_with_comparisons(
        self,
        name: str,
        comparisons: Comparisons,
        instructions: str = None,
        project: str = None,
        version: str = None,
    ):
        request_data = {
            "experiment": {
                "name": name,
                "instructions": instructions,
                "project": project,
                "version": version
            },
            "comparisons": comparisons
        }

        return self._send_create_experiment_request(request_data=request_data)

    def create_binary_evaluation_experiment(
        self,
        name: str,
        samples: list,
        instructions: Optional[str] = None,
        project: Optional[str] = None,
    ) -> None:
        if not name:
            raise ValueError("Experiment name is required.")

        request_data = {
            "experiment": {
                "name": name,
                "instructions": instructions,
                "project": project,
            },
            "samples": samples,
        }

        return self._send_create_experiment_request(request_data=request_data)

    def create_bake_off_evaluation_experiment(
        self,
        name: str,
        comparisons: list,
        instructions: Optional[str] = None,
        project: Optional[str] = None,
    ) -> None:
        if not name:
            raise ValueError("Experiment name is required.")

        request_data = {
            "experiment": {
                "name": name,
                "instructions": instructions,
                "project": project,
            },
            "comparisons": comparisons,
        }

        return self._send_create_experiment_request(request_data=request_data)

    def log_binary_sample(self, experiment_id: int, sample: BinarySample) -> None:
        endpoint = f"{self.base_endpoint}/{experiment_id}/samples?apiKey={self.api_key}"

        try:
            response = requests.post(endpoint, json=sample.dict(by_alias=True))
            response.raise_for_status()
            return response
        except MelodiAPIError as e:
            raise MelodiAPIError(e)

    def log_bake_off_comparison(
        self,
        experiment_id: int,
        sample_1: BakeoffSample,
        sample_2: BakeoffSample,
    ) -> None:
        comparison = {"samples": [sample_1.dict(by_alias=True), sample_2.dict(by_alias=True)]}
        endpoint = f"{self.base_endpoint}/{experiment_id}/comparisons?apiKey={self.api_key}"

        try:
            response = requests.post(
                endpoint, headers=self._get_headers(), json=comparison
            )
            response.raise_for_status()
            return response
        except MelodiAPIError as e:
            raise MelodiAPIError(e)

    def make_shareable(self, experiment_id: int) -> Optional[str]:
        url = f"{self.base_endpoint}/{experiment_id}/shareable-link?apiKey={self.api_key}"

        response = requests.post(url)

        return (
            response.json().get("shareableLink")
            if response.status_code == 200
            else None
        )





import logging
from typing import List

import requests
from pydantic import parse_obj_as

from melodi.base_client import BaseClient
from melodi.exceptions import MelodiAPIError
from melodi.logging import _log_melodi_http_errors
from melodi.projects.data_models import ProjectResponse


class ProjectsClient(BaseClient):
    def __init__(self, base_url: str, api_key: str):
        self.api_key = api_key
        self.base_url = base_url

        self.base_endpoint = self.base_url + "/api/external/projects"
        self.endpoint = self.base_endpoint + f"?apiKey={self.api_key}"

        self.logger = logging.getLogger(__name__)

    def get(self) -> List[ProjectResponse]:
        try:
            response = requests.request("GET", self.endpoint)

            _log_melodi_http_errors(self.logger, response)
            response.raise_for_status()

            return parse_obj_as(List[ProjectResponse], response.json())
        except MelodiAPIError as e:
            raise MelodiAPIError(e)

    def get_by_name(self, name: str) -> ProjectResponse:
        try:
            response = requests.request("GET", f"{self.endpoint}&name={name}")

            _log_melodi_http_errors(self.logger, response)
            response.raise_for_status()

            projects = parse_obj_as(List[ProjectResponse], response.json())

            if (len(projects) > 0):
                return projects[0]
            else:
                return None

        except MelodiAPIError as e:
            raise MelodiAPIError(e)

    def create(self, name: str) -> ProjectResponse:
        try:
            response = requests.post(
                self.endpoint, headers=self._get_headers(), json={"name": name}
            )

            _log_melodi_http_errors(self.logger, response)
            response.raise_for_status()
            return parse_obj_as(ProjectResponse, response.json())
        except MelodiAPIError as e:
            raise MelodiAPIError(e)



import logging
import os
from typing import Optional

from melodi.experiments.experiments_client import ExperimentsClient
from melodi.feedback.feedback_client import FeedbackClient
from melodi.messages.messages_client import MessagesClient
from melodi.projects.projects_client import ProjectsClient
from melodi.threads.threads_client import ThreadsClient
from melodi.users.user_client import UserClient

from .exceptions import MelodiAPIError


class MelodiClient:
    def __init__(self, api_key: Optional[str] = None, verbose=False):
        self.api_key = api_key or os.environ.get("MELODI_API_KEY")

        if not self.api_key:
            raise MelodiAPIError(
                "API key not found. Set the MELODI_API_KEY environment "
                "variable or pass it as an argument."
            )

        self.base_url = os.environ.get("MELODI_BASE_URL_OVERRIDE") or "https://app.melodi.fyi"

        self.logger = logging.getLogger(__name__)

        self.threads = ThreadsClient(base_url=self.base_url, api_key=self.api_key)
        self.projects = ProjectsClient(base_url=self.base_url, api_key=self.api_key)
        self.feedback = FeedbackClient(base_url=self.base_url, api_key=self.api_key)
        self.users = UserClient(base_url=self.base_url, api_key=self.api_key)
        self.messages = MessagesClient(base_url=self.base_url, api_key=self.api_key)
        self.experiments = ExperimentsClient(base_url=self.base_url, api_key=self.api_key)

        if verbose:
            logging.basicConfig(level=logging.INFO)
        else:
            logging.basicConfig(level=logging.ERROR)

    @staticmethod
    def _get_headers():
        return {"Content-Type": "application/json"}

